import socket
import telemetry_pb2 as telemetry_pb2
import random
import pygame
import math
import time
from google.protobuf.text_format import MessageToString

# Initialize pygame
pygame.init()

# Screen dimensions
WIDTH, HEIGHT = 500, 500

# Colors
WHITE = (255, 255, 255)
YELLOW = (255, 255, 0)
BLUE = (50, 120, 255)
RED = (255, 0, 0)  # Color for the trajectory

# Screen setup
screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Vitirover Robot Control Keyboard")

# Function to set speed based on pressed key
def set_speed_from_key():
    global low_order  # Declare low_order as global to modify it

    keys = pygame.key.get_pressed()
    v = 0
    omega = 0.001

    if keys[pygame.K_UP]:
        omega = 0.0
        v = 0.5 
    elif keys[pygame.K_DOWN]:
        omega = 0.0
        v = -0.5
    elif keys[pygame.K_LEFT]:
        omega = -1.0
        v = 0.5
    elif keys[pygame.K_RIGHT]:
        omega = 1.0
        v = 0.5
    
    dt = 1/60.0
    e = 0.340  # Track width (m)
    r = 0.088  # Wheel radius (m)
    x = 0.400
    R = x/(math.tan(omega*dt)+0.0001)

    wG = (v/r) + omega * e/(2*r)
    wH = (v/r) - omega * e/(2*r)
    wI = (v/r) * (1 - e/(2*R))
    wJ = (v/r) * (1 + e/(2*R))

    low_order.front_left_speed = int(wI * 15)
    print("wI: ", low_order.front_left_speed)
    low_order.front_right_speed = int(wJ * 15)
    print("wJ: ", low_order.front_right_speed)
    low_order.back_left_speed = int(wG * 15)
    print("wG: ", low_order.back_left_speed)
    low_order.back_right_speed = int(wH * 15)
    print("wH: ", low_order.back_right_speed)

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock.bind(("192.168.1.123", 5005))
sock.setblocking(0)

low_order = telemetry_pb2.VitiroverLowLevelOrder()  # Initialize here for global accessibility
ControlMode = 1
while True:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit()

    set_speed_from_key()

    order = telemetry_pb2.VitiroverOrder()
    order.low_level_order.CopyFrom(low_order)

    data = order.SerializeToString()
    sock.sendto(data, ("192.168.1.42", 5005))
    time.sleep(0.1)
    try:
        data, addr = sock.recvfrom(20000)
        telemetry_data = telemetry_pb2.VitiroverTelemetry()
        telemetry_data.ParseFromString(data)
        #print("Motor Data:", telemetry_data.front_left_wheel)
        print("timeout")
    except BlockingIOError:
        print("erreur on sendto")
        pass
    
    telemetry_data = None
    while True:
        try:
            data, addr = sock.recvfrom(20000)  # Buffer size
            telemetry_data = telemetry_pb2.VitiroverTelemetry()
        except BlockingIOError:
            # Plus de messages dans le buffer, sortir de la boucle
            break
    if telemetry_data != None:
        telemetry_data.ParseFromString(data)
        # Afficher quelques valeurs
        print("Motor Data:", telemetry_data.front_left_wheel)

    time.sleep(0.1)